package com.gic.cloud.web.controller;

import com.gic.cloud.constants.FunctionEnum;
import com.gic.cloud.dto.AccountGroupDTO;
import com.gic.cloud.dto.UserDTO;
import com.gic.cloud.qo.UserQO;
import com.gic.cloud.service.AccountGroupApiService;
import com.gic.cloud.service.FunctionApiService;
import com.gic.cloud.service.UserApiService;
import com.gic.cloud.web.vo.AccountGroupVO;
import com.gic.cloud.web.vo.FunctionListVO;
import com.gic.cloud.web.vo.UserVO;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/user")
public class UserController {
    private final static Logger LOGGER = LogManager.getLogger(UserController.class);

    @Autowired
    private AccountGroupApiService accountGroupApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private FunctionApiService functionApiService;

    @RequestMapping("/save-account-group")
    public RestResponse saveAccountGroup(AccountGroupDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(accountGroupApiService.saveAccountGroup(dto));
    }

    @RequestMapping("/edit-account-group")
    public RestResponse editAccountGroup(AccountGroupDTO dto) {
        return ResultControllerUtils.commonResult(accountGroupApiService.editAccountGroup(dto));
    }

    @RequestMapping("/sort-account-group")
    public RestResponse sortAccountGroup(Integer accountGroupId, Integer targetSort) {
        return ResultControllerUtils.commonResult(accountGroupApiService.sortAccountGroup(accountGroupId, targetSort));
    }

    @RequestMapping("/list-account-group")
    public RestResponse listAccountGroup() {
        return ResultControllerUtils.commonResult(accountGroupApiService
                .listAccountGroupOfCountMember(UserDetailUtils.getUserDetail().getEnterpriseId()), AccountGroupVO.class);
    }

    @RequestMapping("/delete-account-group")
    public RestResponse deleteAccountGroup(Integer accountGroupId) {
        return ResultControllerUtils.commonResult(accountGroupApiService.deleteAccountGroup(accountGroupId));
    }

    @RequestMapping("/save-user")
    public RestResponse saveUser(UserDTO dto) {
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(userApiService.saveUser(dto));
    }

    @RequestMapping("/edit-user")
    public RestResponse editUser(UserDTO dto) {
        return ResultControllerUtils.commonResult(userApiService.editUser(dto));
    }

    @RequestMapping("/delete-user")
    public RestResponse deleteUser(Integer userId) {
        return ResultControllerUtils.commonResult(userApiService.deleteUser(userId));
    }

    @RequestMapping("/list-function-component")
    public RestResponse listFunctionComponent() {
        List<Map<String, String>> list = new ArrayList<>(8);
        for (FunctionEnum functionEnum : FunctionEnum.values()) {
            Map<String, String> map = new HashMap<>(2);
            map.put(functionEnum.getCode(), functionEnum.getName());
            list.add(map);
        }
        return RestResponse.success(list);
    }

    @RequestMapping("/list-function")
    public RestResponse listFunction(String functionName) {
        return ResultControllerUtils.commonResult(functionApiService
                .listFunction(UserDetailUtils.getUserDetail().getEnterpriseId(), functionName), FunctionListVO.class);
    }

    @RequestMapping("/page-user")
    public RestResponse pageUser(UserQO userQO) {
        userQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonPageResult(userApiService.pageUser(userQO), UserVO.class);
    }

    @RequestMapping("/bulk-transfer-user-to-new-account-group")
    public RestResponse bulkTransferAccountGroup(String userIds, Integer targetAccountGroupId) {
        List<Integer> userIdList = Arrays.stream(userIds.split(","))
                .filter(e -> StringUtils.isNumeric(e))
                .mapToInt(e -> Integer.parseInt(e))
                .boxed()
                .collect(Collectors.toList());
        return ResultControllerUtils.commonResult(userApiService.bulkTransferAccountGroup(userIdList, targetAccountGroupId));
    }
}
